@@ -10,12 +10,12 @@ See COPYING for license information.
 
 import  socket
 import  os
-from    urllib    import quote
+from    urllib    import urlencode
 from    httplib   import HTTPSConnection, HTTPConnection, HTTPException
 from    container import Container, ContainerResults
 from    utils     import unicode_quote, parse_url, THTTPConnection, THTTPSConnection
 from    errors    import ResponseError, NoSuchContainer, ContainerNotEmpty, \
-                         InvalidContainerName, CDNNotEnabled
+                         InvalidContainerName, CDNNotEnabled, ContainerExists
 from    Queue     import Queue, Empty, Full
 from    time      import time
 import  consts
@@ -79,7 +79,7 @@ class Connection(object):
             authurl = kwargs.get('authurl', consts.us_authurl)
             if username and api_key and authurl:
                 self.auth = Authentication(username, api_key, authurl=authurl,
-                            useragent=self.user_agent)
+                            useragent=self.user_agent, timeout=self.timeout)
             else:
                 raise TypeError("Incorrect or invalid arguments supplied")
 
@@ -169,10 +169,7 @@ class Connection(object):
                  (self.uri.rstrip('/'), '/'.join([unicode_quote(i) for i in path]))
 
         if isinstance(parms, dict) and parms:
-            query_args = \
-                ['%s=%s' % (quote(x),
-                            quote(str(y))) for (x, y) in parms.items()]
-            path = '%s?%s' % (path, '&'.join(query_args))
+            path = '%s?%s' % (path, urlencode(parms))
 
         headers = {'Content-Length': str(len(data)),
                    'User-Agent': self.user_agent,
@@ -232,7 +229,7 @@ class Connection(object):
                 len(container_name) > consts.container_name_limit:
             raise InvalidContainerName(container_name)
 
-    def create_container(self, container_name):
+    def create_container(self, container_name, error_on_existing=False):
         """
         Given a container name, returns a L{Container} item, creating a new
         Container if one does not already exist.
@@ -242,6 +239,9 @@ class Connection(object):
 
         @param container_name: name of the container to create
         @type container_name: str
+        @param error_on_existing: raise ContainerExists if container already
+        exists
+        @type error_on_existing: bool
         @rtype: L{Container}
         @return: an object representing the newly created container
         """
@@ -251,6 +251,8 @@ class Connection(object):
         buff = response.read()
         if (response.status < 200) or (response.status > 299):
             raise ResponseError(response.status, response.reason)
+        if error_on_existing and (response.status == 202):
+            raise ContainerExists(container_name)
         return Container(self, container_name)
 
     def delete_container(self, container_name):
@@ -267,10 +269,11 @@ class Connection(object):
         self._check_container_name(container_name)
 
         response = self.make_request('DELETE', [container_name])
-        buff = response.read()
 
         if (response.status == 409):
             raise ContainerNotEmpty(container_name)
+        elif (response.status == 404):
+            raise NoSuchContainer
         elif (response.status < 200) or (response.status > 299):
             raise ResponseError(response.status, response.reason)
 
